#    Copyright 2015 Mirantis, Inc.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.


from oslo_messaging._drivers.zmq_driver.client.publishers.dealer \
    import zmq_dealer_call_publisher
from oslo_messaging._drivers.zmq_driver.client.publishers.dealer \
    import zmq_dealer_publisher
from oslo_messaging._drivers.zmq_driver.client.publishers \
    import zmq_push_publisher
from oslo_messaging._drivers.zmq_driver.client import zmq_client_base
from oslo_messaging._drivers.zmq_driver import zmq_async
from oslo_messaging._drivers.zmq_driver import zmq_names

zmq = zmq_async.import_zmq()


class ZmqClient(zmq_client_base.ZmqClientBase):

    def __init__(self, conf, matchmaker=None, allowed_remote_exmods=None):

        default_publisher = zmq_dealer_publisher.DealerPublisher(
            conf, matchmaker)

        fanout_publisher = zmq_dealer_publisher.DealerPublisherLight(
            conf, matchmaker) if conf.use_pub_sub else default_publisher

        super(ZmqClient, self).__init__(
            conf, matchmaker, allowed_remote_exmods,
            publishers={
                zmq_names.CALL_TYPE:
                    zmq_dealer_call_publisher.DealerCallPublisher(
                        conf, matchmaker),

                zmq_names.CAST_TYPE:
                    zmq_push_publisher.PushPublisher(conf, matchmaker),

                # Here use DealerPublisherLight for sending request to proxy
                # which finally uses PubPublisher to send fanout in case of
                # 'use_pub_sub' option configured.
                zmq_names.CAST_FANOUT_TYPE: fanout_publisher,

                zmq_names.NOTIFY_TYPE: fanout_publisher,

                "default": default_publisher
            }
        )




# Copyright 2012 OpenStack Foundation
# Copyright 2013 Red Hat, Inc.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import oslo_messaging
from oslo_messaging.tests import utils as test_utils


class TestExpectedExceptions(test_utils.BaseTestCase):

    def test_exception(self):
        e = None
        try:
            try:
                raise ValueError()
            except Exception:
                raise oslo_messaging.ExpectedException()
        except oslo_messaging.ExpectedException as e:
            self.assertIsInstance(e, oslo_messaging.ExpectedException)
            self.assertTrue(hasattr(e, 'exc_info'))
            self.assertIsInstance(e.exc_info[1], ValueError)

    def test_decorator_expected(self):
        class FooException(Exception):
            pass

        @oslo_messaging.expected_exceptions(FooException)
        def naughty():
            raise FooException()

        self.assertRaises(oslo_messaging.ExpectedException, naughty)

    def test_decorator_expected_subclass(self):
        class FooException(Exception):
            pass

        class BarException(FooException):
            pass

        @oslo_messaging.expected_exceptions(FooException)
        def naughty():
            raise BarException()

        self.assertRaises(oslo_messaging.ExpectedException, naughty)

    def test_decorator_unexpected(self):
        class FooException(Exception):
            pass

        @oslo_messaging.expected_exceptions(FooException)
        def really_naughty():
            raise ValueError()

        self.assertRaises(ValueError, really_naughty)



#    Copyright 2015 Mirantis, Inc.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.


import logging

from oslo_messaging._drivers import base
from oslo_messaging._drivers import common as rpc_common
from oslo_messaging._drivers.zmq_driver import zmq_async
from oslo_messaging._drivers.zmq_driver import zmq_names


LOG = logging.getLogger(__name__)

zmq = zmq_async.import_zmq()


class ZmqIncomingRequest(base.RpcIncomingMessage):

    def __init__(self, socket, rep_id, request, poller):
        super(ZmqIncomingRequest, self).__init__(request.context,
                                                 request.message)
        self.reply_socket = socket
        self.reply_id = rep_id
        self.request = request
        self.received = None
        self.poller = poller

    def reply(self, reply=None, failure=None, log_failure=True):
        if failure is not None:
            failure = rpc_common.serialize_remote_exception(failure,
                                                            log_failure)
        message_reply = {zmq_names.FIELD_TYPE: zmq_names.REPLY_TYPE,
                         zmq_names.FIELD_REPLY: reply,
                         zmq_names.FIELD_FAILURE: failure,
                         zmq_names.FIELD_LOG_FAILURE: log_failure,
                         zmq_names.FIELD_MSG_ID: self.request.message_id}

        LOG.debug("Replying %s", (str(self.request.message_id)))

        self.received = True
        self.reply_socket.send(self.reply_id, zmq.SNDMORE)
        self.reply_socket.send(b'', zmq.SNDMORE)
        self.reply_socket.send_pyobj(message_reply)
        self.poller.resume_polling(self.reply_socket)

    def requeue(self):
        """Requeue is not supported"""
